@@ -367,6 +367,7 @@ pub struct Options {
     /// The end of the segment we actually process
     pub extraction_end: Option<Timestamp>,
     pub print_file_reports: bool,
+    pub report_format: Option<String>,
     /// Contains the settings for the 608 decoder.
     pub settings_608: Decoder608Settings,
     /// Same for 708 decoder
@@ -466,8 +467,9 @@ pub struct Options {
     pub ocr_line_split: bool,
     /// If true, use character blacklist to prevent common OCR errors (e.g. | vs I)
     pub ocr_blacklist: bool,
-    /// The name of the language stream for MKV
-    pub mkvlang: Option<Language>,
+    /// Language filter for MKV subtitle tracks.
+    /// Accepts comma-separated ISO 639-2 codes (e.g., "eng,fre") or BCP 47 tags (e.g., "en-US,fr-CA").
+    pub mkvlang: Option<super::MkvLangFilter>,
     /// If true, the video stream will be processed even if we're using a different one for subtitles.
     pub analyze_video_stream: bool,
 
@@ -630,6 +632,7 @@ impl Default for Options {
             segment_on_key_frames_only: Default::default(),
             scc_framerate: 0,           // 0 = 29.97fps (default)
             scc_accurate_timing: false, // Off by default for backwards compatibility (issue #1120)
+            report_format: None,
             debug_mask: DebugMessageMask::new(
                 DebugMessageFlag::GENERIC_NOTICE,
                 DebugMessageFlag::VERBOSE,
