@@ -18,6 +18,7 @@ use lib_ccxr::common::DtvccServiceCharset;
 use lib_ccxr::common::EncoderConfig;
 use lib_ccxr::common::EncodersTranscriptFormat;
 use lib_ccxr::common::Language;
+use lib_ccxr::common::MkvLangFilter;
 use lib_ccxr::common::Options;
 use lib_ccxr::common::OutputFormat;
 use lib_ccxr::common::SelectCodec;
@@ -80,6 +81,11 @@ pub unsafe fn copy_from_rust(ccx_s_options: *mut ccx_s_options, options: Options
     (*ccx_s_options).extraction_start = options.extraction_start.to_ctype();
     (*ccx_s_options).extraction_end = options.extraction_end.to_ctype();
     (*ccx_s_options).print_file_reports = options.print_file_reports as _;
+    // Report output format (e.g. "json")
+    if let Some(ref fmt) = options.report_format {
+        (*ccx_s_options).report_format =
+            replace_rust_c_string((*ccx_s_options).report_format, fmt.as_str());
+    }
     // Preserve the original C-managed report pointer to avoid dangling pointer issues.
     let saved_608_report = (*ccx_s_options).settings_608.report;
     (*ccx_s_options).settings_608 = options.settings_608.to_ctype();
@@ -183,9 +189,9 @@ pub unsafe fn copy_from_rust(ccx_s_options: *mut ccx_s_options, options: Options
     (*ccx_s_options).ocr_quantmode = options.ocr_quantmode as _;
     (*ccx_s_options).ocr_line_split = options.ocr_line_split as _;
     (*ccx_s_options).ocr_blacklist = options.ocr_blacklist as _;
-    if let Some(mkvlang) = options.mkvlang {
+    if let Some(ref mkvlang) = options.mkvlang {
         (*ccx_s_options).mkvlang =
-            replace_rust_c_string((*ccx_s_options).mkvlang, mkvlang.to_ctype().as_str());
+            replace_rust_c_string((*ccx_s_options).mkvlang, mkvlang.as_raw_str());
     }
     (*ccx_s_options).analyze_video_stream = options.analyze_video_stream as _;
     (*ccx_s_options).hardsubx_ocr_mode = options.hardsubx_ocr_mode.to_ctype();
@@ -211,11 +217,9 @@ pub unsafe fn copy_from_rust(ccx_s_options: *mut ccx_s_options, options: Options
             replace_rust_c_string((*ccx_s_options).udpaddr, &options.udpaddr.clone().unwrap());
     }
     (*ccx_s_options).udpport = options.udpport as _;
-    if options.tcpport.is_some() {
-        (*ccx_s_options).tcpport = replace_rust_c_string(
-            (*ccx_s_options).tcpport,
-            &options.tcpport.unwrap().to_string(),
-        );
+    if let Some(tcpport) = options.tcpport {
+        (*ccx_s_options).tcpport =
+            replace_rust_c_string((*ccx_s_options).tcpport, &tcpport.to_string());
     }
     if options.tcp_password.is_some() {
         (*ccx_s_options).tcp_password = replace_rust_c_string(
@@ -235,11 +239,9 @@ pub unsafe fn copy_from_rust(ccx_s_options: *mut ccx_s_options, options: Options
             &options.srv_addr.clone().unwrap(),
         );
     }
-    if options.srv_port.is_some() {
-        (*ccx_s_options).srv_port = replace_rust_c_string(
-            (*ccx_s_options).srv_port,
-            &options.srv_port.unwrap().to_string(),
-        );
+    if let Some(srv_port) = options.srv_port {
+        (*ccx_s_options).srv_port =
+            replace_rust_c_string((*ccx_s_options).srv_port, &srv_port.to_string());
     }
     (*ccx_s_options).noautotimeref = options.noautotimeref as _;
     (*ccx_s_options).input_source = options.input_source as _;
@@ -253,15 +255,12 @@ pub unsafe fn copy_from_rust(ccx_s_options: *mut ccx_s_options, options: Options
     // Subsequent calls from ccxr_demuxer_open/close should NOT modify inputfile because
     // C code holds references to those strings throughout processing.
     // Freeing them would cause use-after-free and double-free errors.
-    if options.inputfile.is_some() && (*ccx_s_options).inputfile.is_null() {
-        (*ccx_s_options).inputfile = string_to_c_chars(options.inputfile.clone().unwrap());
-        (*ccx_s_options).num_input_files = options
-            .inputfile
-            .as_ref()
-            .unwrap()
-            .iter()
-            .filter(|s| !s.is_empty())
-            .count() as _;
+    if let Some(ref inputfile) = options.inputfile {
+        if (*ccx_s_options).inputfile.is_null() {
+            (*ccx_s_options).inputfile = string_to_c_chars(inputfile.clone());
+            (*ccx_s_options).num_input_files =
+                inputfile.iter().filter(|s| !s.is_empty()).count() as _;
+        }
     }
     (*ccx_s_options).demux_cfg = options.demux_cfg.to_ctype();
     // Only set enc_cfg on the first call (when output_filename is null).
@@ -329,6 +328,7 @@ pub unsafe fn copy_to_rust(ccx_s_options: *const ccx_s_options) -> Options {
             .expect("Invalid extraction end time"),
         ),
         print_file_reports: (*ccx_s_options).print_file_reports != 0,
+        report_format: None,
         // Handle settings_608 and settings_dtvcc - assuming FromCType trait is implemented for these
         settings_608: Decoder608Settings::from_ctype((*ccx_s_options).settings_608)
             .unwrap_or(Decoder608Settings::default()),
@@ -350,6 +350,10 @@ pub unsafe fn copy_to_rust(ccx_s_options: *const ccx_s_options) -> Options {
         ..Default::default()
     };
 
+    if !(*ccx_s_options).report_format.is_null() {
+        options.report_format = Some(c_char_to_string((*ccx_s_options).report_format));
+    }
+
     // Handle sentence_cap_file (C string to PathBuf)
     if !(*ccx_s_options).sentence_cap_file.is_null() {
         options.sentence_cap_file =
@@ -425,12 +429,10 @@ pub unsafe fn copy_to_rust(ccx_s_options: *const ccx_s_options) -> Options {
     options.ocr_line_split = (*ccx_s_options).ocr_line_split != 0;
     options.ocr_blacklist = (*ccx_s_options).ocr_blacklist != 0;
 
-    // Handle mkvlang (C string to Option<Language>)
+    // Handle mkvlang (C string to Option<MkvLangFilter>)
     if !(*ccx_s_options).mkvlang.is_null() {
-        options.mkvlang = Some(
-            Language::from_str(&c_char_to_string((*ccx_s_options).mkvlang))
-                .expect("Invalid language"),
-        )
+        let lang_str = c_char_to_string((*ccx_s_options).mkvlang);
+        options.mkvlang = MkvLangFilter::new(&lang_str).ok();
     }
 
     options.analyze_video_stream = (*ccx_s_options).analyze_video_stream != 0;
