@@ -24,8 +24,10 @@ const CCX_DTVCC_MAX_PACKET_LENGTH: u8 = 128;
 const CCX_DTVCC_NO_LAST_SEQUENCE: i32 = -1;
 const CCX_DTVCC_SCREENGRID_ROWS: u8 = 75;
 const CCX_DTVCC_SCREENGRID_COLUMNS: u8 = 210;
-const CCX_DTVCC_MAX_ROWS: u8 = 15;
-const CCX_DTVCC_MAX_COLUMNS: u8 = 32 * 2;
+pub(crate) const CCX_DTVCC_MAX_ROWS: u8 = 15;
+pub(crate) const CCX_DTVCC_MAX_COLUMNS: u8 = 32 * 2;
+
+pub use window::{alloc_row, dealloc_row};
 
 /// Context required for processing 708 data
 pub struct Dtvcc<'a> {
