@@ -7,7 +7,6 @@ use crate::demuxer::common_types::{
 };
 use lib_ccxr::common::{Codec, Options, StreamMode, StreamType};
 use lib_ccxr::time::Timestamp;
-use std::alloc::{alloc_zeroed, Layout};
 use std::ffi::CStr;
 use std::os::raw::{c_char, c_int, c_uchar, c_uint, c_void};
 
@@ -18,12 +17,15 @@ const POISON_PTR_PATTERN: usize = 0xcdcdcdcdcdcdcdcd;
 #[cfg(target_pointer_width = "32")]
 const POISON_PTR_PATTERN: usize = 0xcdcdcdcd;
 
-// External C function declarations
 extern "C" {
-    fn activity_input_file_closed();
-    fn close(fd: c_int) -> c_int;
+    pub(crate) fn activity_input_file_closed();
+    pub(crate) fn close(fd: c_int) -> c_int;
+    pub(crate) fn malloc(size: usize) -> *mut c_void;
+    pub(crate) fn calloc(nmemb: usize, size: usize) -> *mut c_void;
 }
 
+use crate::ffi_alloc;
+
 pub fn copy_c_array_to_rust_vec(
     c_bytes: &[u8; crate::demuxer::common_types::ARRAY_SIZE],
 ) -> Vec<u8> {
@@ -98,61 +100,89 @@ pub unsafe fn copy_demuxer_from_rust_to_c(c_demuxer: *mut ccx_demuxer, rust_demu
     c.global_timestamp_inited = rust_demuxer.global_timestamp_inited.millis() as c_int;
 
     // PID buffers - extra defensive version
+    // We iterate through all possible PIDs (up to 8191 for PSI) to ensure state synchronization.
+    // CRITICAL: We must free existing pointers in the C structure before overwriting them
+    // to prevent massive memory leaks during the demuxing process, as this function
+    // is called repeatedly to sync state between Rust and C.
     let pid_buffers_len = rust_demuxer.pid_buffers.len().min(8191);
-    for i in 0..pid_buffers_len {
-        let pid_buffer = rust_demuxer.pid_buffers[i];
-        if !pid_buffer.is_null() {
-            // Try to safely access the pointer
-            match std::panic::catch_unwind(|| unsafe { &*pid_buffer }) {
-                Ok(rust_psi) => {
-                    let c_psi = unsafe { rust_psi.to_ctype() };
-                    let c_ptr = Box::into_raw(Box::new(c_psi));
-                    c.PID_buffers[i] = c_ptr;
-                }
-                Err(_) => {
-                    // Pointer was invalid, set to null
-                    eprintln!("Warning: Invalid PID buffer pointer at index {i}");
-                    c.PID_buffers[i] = std::ptr::null_mut();
+    for i in 0..8191 {
+        // Free existing pointer if any.
+        // SAFETY: We use C's free to be compatible with memory that might be allocated by C.
+        // We also check for POISON_PTR_PATTERN for safety in debug builds.
+        if !c.PID_buffers[i].is_null() && c.PID_buffers[i] as usize != POISON_PTR_PATTERN {
+            unsafe {
+                ffi_alloc::c_free(c.PID_buffers[i]);
+                c.PID_buffers[i] = std::ptr::null_mut();
+            }
+        }
+
+        if i < pid_buffers_len {
+            let pid_buffer = rust_demuxer.pid_buffers[i];
+            if !pid_buffer.is_null() {
+                // Try to safely access the pointer using catch_unwind to prevent
+                // a panic in Rust from crashing the entire C application.
+                // This is a defensive measure for FFI robustness.
+                match std::panic::catch_unwind(|| unsafe { &*pid_buffer }) {
+                    Ok(rust_psi) => {
+                        let c_psi = unsafe { rust_psi.to_ctype() };
+                        let c_ptr =
+                            unsafe { malloc(std::mem::size_of::<crate::bindings::PSI_buffer>()) }
+                                as *mut crate::bindings::PSI_buffer;
+                        if !c_ptr.is_null() {
+                            unsafe {
+                                std::ptr::write(c_ptr, c_psi);
+                            }
+                            c.PID_buffers[i] = c_ptr;
+                        }
+                    }
+                    Err(_) => {
+                        // Pointer was invalid, log and skip
+                        eprintln!("Warning: Invalid PID buffer pointer at index {i}");
+                    }
                 }
             }
-        } else {
-            c.PID_buffers[i] = std::ptr::null_mut();
         }
     }
 
-    // Clear remaining slots if rust array is smaller than C array
-    for i in pid_buffers_len..8191 {
-        c.PID_buffers[i] = std::ptr::null_mut();
-    }
-
     // PIDs programs - extra defensive version
+    // Similar to PID_buffers, we manage ownership of PMT entries.
+    // We check for POISON_PTR_PATTERN to avoid freeing uninitialized memory in debug builds.
     let pids_programs_len = rust_demuxer.pids_programs.len().min(65536);
-    for i in 0..pids_programs_len {
-        let pmt_entry = rust_demuxer.pids_programs[i];
-        if !pmt_entry.is_null() {
-            // Try to safely access the pointer
-            match std::panic::catch_unwind(|| unsafe { &*pmt_entry }) {
-                Ok(rust_pmt) => {
-                    let c_pmt = unsafe { rust_pmt.to_ctype() };
-                    let c_ptr = Box::into_raw(Box::new(c_pmt));
-                    c.PIDs_programs[i] = c_ptr;
-                }
-                Err(_) => {
-                    // Pointer was invalid, set to null
-                    eprintln!("Warning: Invalid PMT entry pointer at index {i}");
-                    c.PIDs_programs[i] = std::ptr::null_mut();
+    for i in 0..65536 {
+        // Free existing pointer if any and it's not a poison pattern.
+        // SAFETY: We use C's free to be compatible with memory that might be allocated by C.
+        if !c.PIDs_programs[i].is_null() && c.PIDs_programs[i] as usize != POISON_PTR_PATTERN {
+            unsafe {
+                ffi_alloc::c_free(c.PIDs_programs[i]);
+                c.PIDs_programs[i] = std::ptr::null_mut();
+            }
+        }
+
+        if i < pids_programs_len {
+            let pmt_entry = rust_demuxer.pids_programs[i];
+            if !pmt_entry.is_null() {
+                // Safely convert and move ownership to C
+                match std::panic::catch_unwind(|| unsafe { &*pmt_entry }) {
+                    Ok(rust_pmt) => {
+                        let c_pmt = unsafe { rust_pmt.to_ctype() };
+                        let c_ptr =
+                            unsafe { malloc(std::mem::size_of::<crate::bindings::PMT_entry>()) }
+                                as *mut crate::bindings::PMT_entry;
+                        if !c_ptr.is_null() {
+                            unsafe {
+                                std::ptr::write(c_ptr, c_pmt);
+                            }
+                            c.PIDs_programs[i] = c_ptr;
+                        }
+                    }
+                    Err(_) => {
+                        eprintln!("Warning: Invalid PMT entry pointer at index {i}");
+                    }
                 }
             }
-        } else {
-            c.PIDs_programs[i] = std::ptr::null_mut();
         }
     }
 
-    // Clear remaining slots if rust array is smaller than C array
-    for i in pids_programs_len..65536 {
-        c.PIDs_programs[i] = std::ptr::null_mut();
-    }
-
     // PIDs seen array
     for (i, &val) in rust_demuxer.pids_seen.iter().take(65536).enumerate() {
         c.PIDs_seen[i] = val as c_int;
@@ -265,7 +295,15 @@ pub unsafe fn copy_demuxer_from_c_to_rust(ccx: *const ccx_demuxer) -> CcxDemuxer
             if buffer_ptr.is_null() {
                 None
             } else {
-                Some(Box::into_raw(Box::new(PSIBuffer::from_ctype(*buffer_ptr)?)))
+                let rust_item = PSIBuffer::from_ctype(*buffer_ptr)?;
+                let rust_ptr =
+                    unsafe { malloc(std::mem::size_of::<PSIBuffer>()) } as *mut PSIBuffer;
+                if !rust_ptr.is_null() {
+                    unsafe {
+                        std::ptr::write(rust_ptr, rust_item);
+                    }
+                }
+                Some(rust_ptr)
             }
         })
         .collect::<Vec<_>>();
@@ -276,7 +314,14 @@ pub unsafe fn copy_demuxer_from_c_to_rust(ccx: *const ccx_demuxer) -> CcxDemuxer
             if buffer_ptr.is_null() || buffer_ptr as usize == POISON_PTR_PATTERN {
                 None
             } else {
-                Some(Box::into_raw(Box::new(PMTEntry::from_ctype(*buffer_ptr)?)))
+                let rust_item = PMTEntry::from_ctype(*buffer_ptr)?;
+                let rust_ptr = unsafe { malloc(std::mem::size_of::<PMTEntry>()) } as *mut PMTEntry;
+                if !rust_ptr.is_null() {
+                    unsafe {
+                        std::ptr::write(rust_ptr, rust_item);
+                    }
+                }
+                Some(rust_ptr)
             }
         })
         .collect::<Vec<_>>();
@@ -367,8 +412,7 @@ pub unsafe fn copy_demuxer_from_c_to_rust(ccx: *const ccx_demuxer) -> CcxDemuxer
 ///
 /// This function is unsafe because we are calling a C struct and using alloc_zeroed to initialize it.
 pub unsafe fn alloc_new_demuxer() -> *mut ccx_demuxer {
-    let layout = Layout::new::<ccx_demuxer>();
-    let ptr = alloc_zeroed(layout) as *mut ccx_demuxer;
+    let ptr = calloc(1, std::mem::size_of::<ccx_demuxer>()) as *mut ccx_demuxer;
 
     if ptr.is_null() {
         panic!("Failed to allocate memory for ccx_demuxer");
@@ -409,6 +453,27 @@ pub unsafe extern "C" fn ccxr_demuxer_close(ctx: *mut ccx_demuxer) {
     }
 }
 
+/// # Safety
+/// This function must be called before the C code frees the demuxer struct.
+/// It frees Rust-allocated resources that C cannot safely free.
+#[no_mangle]
+pub unsafe extern "C" fn ccxr_demuxer_delete(ctx: *mut ccx_demuxer) {
+    if ctx.is_null() {
+        return;
+    }
+    let c = &mut *ctx;
+
+    // Free Rust-allocated filebuffer
+    if !c.filebuffer.is_null() {
+        use crate::file_functions::file::FILEBUFFERSIZE;
+        let _ = Box::from_raw(std::ptr::slice_from_raw_parts_mut(
+            c.filebuffer,
+            FILEBUFFERSIZE,
+        ));
+        c.filebuffer = std::ptr::null_mut();
+    }
+}
+
 // Extern function for ccx_demuxer_isopen
 /// # Safety
 /// This function is unsafe because it dereferences a raw pointer.
